/**************************************************************************************
Author:			David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:		camera_distance.ms
Date:			11/5/11
Description:	This scripts calculates the distance between a camera and an object.
***************************************************************************************/

try (destroyDialog camera_distance_rollout) catch()


struct CameraDistance
(
    camera_objects = #(),
    camera_names = #(),
    
    /**************************************************************************
    Removes the camera distance selection callback
    **************************************************************************/
    fn remove_selection_callback =
    (
        callbacks.removeScripts #selectionSetChanged id:#cameraDistanceCallback
    ), --end fn
    
    /**************************************************************************
    Sets the camera distance selection callback
    **************************************************************************/
    fn add_selection_callback =
    (
        callbacks.addScript #selectionSetChanged "camera_distance_rollout.calculate_distance.pressed()" id:#cameraDistanceCallback
    ), --end fn
    
    /**************************************************************************
    Sorts items by name
    **************************************************************************/
    fn sort_by_name item_one item_two =
    (
        case of
        (
            (item_one.name < item_two.name): -1
            (item_one.name > item_two.name): 1
            default: 0
        ) --end case
    ), --end fn
    
    /**************************************************************************
    Stores a reference to all scene cameras
    **************************************************************************/
    fn collect_scene_cameras =
    (
        this.camera_objects = for current in cameras where classOf current != Targetobject collect current
        
        if this.camera_objects.count > 0 do
            qsort this.camera_objects sort_by_name
            
        this.camera_names = for current in camera_objects collect current.name
    ) --end fn
) --end struct


(--put script into local scope
    global camera_distance_rollout
	local camera_distance_ini = getDir #plugcfg + "\\CameraDistance.ini"
	local camera_distance_position
	
	rollout camera_distance_rollout "Camera Distance"
	(
        local cam_struct = CameraDistance()
        
		groupbox camera_group "Camera" pos:[5,5] width:350 height:45
			dropDownList camera_dropdown "" pos:[15,23] width:260 items:#()
			button update_camera_button "Update" pos:[285,23] width:60 height:20
			
		groupbox distance_group "Distance" pos:[5,55] width:350 height:65
			label object_label "Object:" pos:[15,75] width:330 height:15
			label distance_label "Distance From Camera:" pos:[15,95] width:115 height:15
			edittext distance_result_text "" pos:[135,94] width:210 height:17 readOnly:true
        
        --Invisible button used for callback
        button calculate_distance "" pos:[5,120] visible:false
			
        /**********************************************************************
        UI open event handler. Prepares the UI for first time open
        **********************************************************************/
		on camera_distance_rollout open do
        (
            cam_struct.remove_selection_callback()
            camera_distance_rollout.update_camera_button.pressed()
            cam_struct.add_selection_callback()
        ) --end on
            
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
		on camera_distance_rollout moved pos do
            setIniSetting camera_distance_ini "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        UI close event handler. Removes the selection callback
        **********************************************************************/
		on camera_distance_rollout close do
            cam_struct.remove_selection_callback()
        
        /**********************************************************************
        Camera dropdown event handler. Updates the distance result based on
        the new camera selection
        **********************************************************************/
		on camera_dropdown selected theItem do
            camera_distance_rollout.calculate_distance.pressed()
        
        /**********************************************************************
        Update camera event handler. Updates the camera list
        **********************************************************************/
		on update_camera_button pressed do
        (
            local camera_names = cam_struct.collect_scene_cameras()
            
            if cam_struct.camera_objects.count > 0 then
            (
                camera_dropdown.height = cam_struct.camera_names.count * 16
                camera_dropdown.items = cam_struct.camera_names
                
                camera_distance_rollout.calculate_distance.pressed()
            ) --end if
            else
            (
                camera_dropdown.height = 16
                camera_dropdown.items = #("No Cameras Found")
            ) --end else
        ) --end on
        
        /**********************************************************************
        Calculate event handler. Updates the distance result. Also used in
        selection callback
        **********************************************************************/
        on calculate_distance pressed do
        (
            if cam_struct.camera_objects.count == 0 do
            (
                object_label.text = "Object: ERROR: Camera not found."
                distance_result_text.text = "ERROR: Camera not found."
                return false
            ) --end if
            
            local current_camera = cam_struct.camera_objects[camera_dropdown.selection]
            
            if (isDeleted current_camera) do
            (
                object_label.text = "Object: ERROR: Camera does not exist."
                distance_result_text.text = "ERROR: Camera does not exist."
                return false
            ) --end if
            
            case of
            (
                (selection.count == 0):(
                                            object_label.text = "Object: Nothing Selected"
                                            distance_result_text.text = "Infinity"
                                        ) --end case 1
                (selection.count == 1):(
                                            object_label.text = "Object: " + selection[1].name
                                            distance_result_text.text = ((distance selection[1].center current_camera.center) as string)
                                        ) --end case 2
                (selection.count > 1):(
                                            object_label.text = "Object: Multiple (using selection center)"
                                            distance_result_text.text = ((distance selection.center current_camera.center) as string)
                                        ) --end case 3
            ) --end case
        ) --end on
	) --end rollout
    
	camera_distance_position = execute (getIniSetting camera_distance_ini "Dialog" "Position")
	if camera_distance_position == OK do
        camera_distance_position = [100,100]
	if camera_distance_position.x > sysinfo.desktopsize.x - 360 do
        camera_distance_position.x = sysinfo.desktopsize.x - 365
	if camera_distance_position.y > sysinfo.desktopsize.y - 185 do
        camera_distance_position.y = sysinfo.desktopsize.y - 185
	if camera_distance_position.x < 0 do camera_distance_position.x = 0 
	if camera_distance_position.y < 0 do camera_distance_position.y = 0
	createDialog camera_distance_rollout 360 125 camera_distance_position.x camera_distance_position.y
) --end local scope